#!/usr/bin/env python3
# -*- encoding=utf-8 -*-
# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""

# server.py
# 从wsgiref模块导入:
from wsgiref.simple_server import make_server
# 导入我们自己编写的application函数:
from dueros.samples.personal_income_tax.BotServer import application
# 创建一个服务器，IP地址为空，端口是8000，处理函数是application:
httpd = make_server('', 8000, application)
print('Serving HTTP on port 8000...')
# 开始监听HTTP请求:
httpd.serve_forever()



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-
# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""

from dueros.Bot import Bot
from dueros.card.TextCard import TextCard

class Bot(Bot):
    def launchRequest(self):
        '''
        打开调用名
        '''
        card = TextCard('欢迎来到猜数字游戏')
        return {
            'card' : card,
            'outputSpeech': r'欢迎来到猜数字游戏'
        }

    def compareNum(self):
        '''
        获取数字槽位值处理
        '''
        rightNum = 10
        num = self.getSlots('sys.number');
        num = int(num)
        if num > rightNum:
            return {
                'outputSpeech' :'您所猜的数字大了'
            }
        elif num < rightNum:
            return {
                'outputSpeech' :'您所猜的数字小了'
            }
        else:
            return {
                'outputSpeech' :'恭喜您猜对了'
            }

    def __init__(self, data):
        super(Bot, self).__init__(data)
        self.addLaunchHandler(self.launchRequest)
        self.addIntentHandler('guess_number', self.compareNum)


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/30

from dueros.Base import Base

class Nlu(Base):
    '''

    '''

    def __init__(self, data):
        super(Nlu, self).__init__()
        self.data = data
        self.askSlot = None
        self.directive = None

    def getIntentName(self, index=0):
        '''
        获取当前的意图intent名
        :return:
        '''
        return self.data[index]['name']

    def setSlot(self, field, value, index=0):
        '''
        设置槽位信息
        desc 设置slot, 如果不存在,新增一个solt
        :param field:   槽位名
        :param value:   槽位值
        :param index:   第几组slot
        :return:
        '''

        if not field:
            return

        slots = self.data[index]['slots']

        if field in slots:
            self.data[index]['slots'][field]['value'] = value
        else:
            self.data[index]['slots'][field] = {
                'name': field,
                'value': value
            }

    def getSlot(self, field, index = 0):

        '''
        获取槽位
        @desc 获取一个槽位slot的值
        :param field:
        :param index:
        :return:
        '''

        '''
            intent:[
                {
                   "slots": {
                        "{{STRING}}": {
                            "name": "{{STRING}}",
                            "value": ["{{STRING}}"],
                            "confirmationStatus": "{{STRING}}"
                        }
                    }
                },{
                .....
                } 
            ]
        '''
        if not field:
            return
        # #此处有坑 文档是values 但是PHP demo是value
        return self.__getSlotValueByKey(field, 'value', index)

    def getSlotConfirmationStatus(self, field, index = 0):
        '''
        槽位确认状态
        :param field:
        :param index:
        :return:    NONE: 未确认；CONFIRMED: 确认；DENIED: 否认
        '''
        return self.__getSlotValueByKey(field, 'confirmationStatus', index)

    def getIntentConfirmationStatus(self, index = 0):
        '''
        获取意图的确认状态
        :param index:
        :return:    NONE: 未确认；CONFIRMED: 确认；DENIED: 否认
        '''
        return self.data[index]['confirmationStatus']

    def __getSlotValueByKey(self, field, subField, index = 0):
        '''
        :param field:
        :param subField:
        :param index:
        :return:
        '''

        if not 'slots' in self.data[index]:
            return ''
        slots = self.data[index]['slots']
        if field in slots:
            return slots[field][subField]
        else:
            return None

    def hasAsked(self):
        '''
        是否询问过用户
        :return:
        '''
        if self.directive:
            return True
        else:
            return False

    def ask(self, slot):
        '''
        询问一个特定的槽位
        :param slot:
        :return:
        '''

        if slot != '' and slot:
            # logging.info('askSlot = %s' % slot)
            self.askSlot = slot
            self.directive = {
                'type': 'Dialog.ElicitSlot',
                'slotToElicit': slot,
                'updatedIntent': self.__getUpdateIntent()
            }
        else:
            return

    def toDirective(self):
        '''
        打包NLU交互协议，返回DuerOS，为第二轮用户回答提供上下文
        在Response 中被调用
        :return:
        '''
        return self.directive

    def __getUpdateIntent(self):
        '''
        构造返回的update intent 数据结构
        :return:
        '''

        if 'slots' in self.data[0]:
            return {
                'name': self.getIntentName(),
                'slots': self.data[0]['slots']
            }
        else:
            return {
                'name': self.getIntentName(),
                'slots': {}
            }

    def toUpdateIntent(self):
        '''
        bot可以修改intent中对应的值，返回给DuerOs更新
        在Response 中被调用
        :return:
        '''

        if self.data[0]:
            return {
                'intent': self.data[0]
            }
        else:
            return {
                'intent': {}
            }

    def setDelegate(self):
        '''
        设置delegate 某个槽位或确认意图
        :return:
        '''
        self.directive = {
            'type': 'Dialog.Delegate',
            'updatedIntent': self.__getUpdateIntent()
        }

    def setConfirmSlot(self, field):
        '''
        设置对一个槽位的确认
        :param field:
        :return:
        '''

        if 'slots' in self.data[0]:
            slots = self.data[0]['slots']
            if field in slots:
                self.directive = {
                    'type': 'Dialog.ConfirmSlot',
                    'slotToConfirm': field,
                    'updatedIntent': self.__getUpdateIntent()
                }

    def setConfirmIntent(self):
        '''
        设置confirm 意图。询问用户是否对意图确认，设置后需要自行返回outputSpeech
        :return:
        '''
        self.directive = {
            'type': 'Dialog.ConfirmIntent',
            'updatedIntent': self.__getUpdateIntent()
        }


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/4/15

"""
    desc:pass
"""


class _Constants:

    class ConstError(TypeError): pass

    class ConstCaseError(ConstError): pass

    def __setattr__(self, name, value):
        if name in self.__dict__:
            raise self.ConstError("can't change const %s" % name)
        if not name.isupper():
            raise self.ConstCaseError('const name "%s" is not all uppercase' % name)
        self.__dict__[name] = value


constants = _Constants()

#日志存放地址
constants.LOG_PATH = './apps/log/dueros'

if __name__ == '__main__':
    print(constants.LOG_PATH)
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from dueros.directive.AudioPlayer.Control.BaseButton import BaseButton

class Button(BaseButton):

    def __init__(self, name):
        super(Button, self).__init__('BUTTON', name)
        self.data['enabled'] = True
        self.data['selected'] = False

    def setEnabled(self, bool):
        self.data['enabled'] = bool

    def setSelected(self, bool):
        self.data['selected'] = bool

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from dueros.directive.AudioPlayer.Control.BaseButton import BaseButton

class PlayerInfo:

    def __init__(self):
        self.data = {}
        self.data['content']={}
        self.data['content']['audioItemType'] = 'AUDIO_TYPE_MUSIC'

    def setAudioItemType(self, type):
        self.data['content']['audioItemType'] = type

    def setTitle(self, title):

        self.data['content']['title'] = title

    def setTitleSubtext1(self, titleSubtext1):
        self.data['content']['titleSubtext1'] = titleSubtext1

    def setTitleSubtext2(self, titleSubtext2):
        self.data['content']['titleSubtext2'] = titleSubtext2

    def setLyric(self, url):
        self.data['content']['lyric']['url'] = url
        self.data['content']['lyric']['format'] = 'FORMAT_LRC'

    def setMediaLengthInMs(self, mediaLengthInMs):
        mediaLengthInMs = int(mediaLengthInMs)
        self.data['content']['mediaLengthInMilliseconds'] = mediaLengthInMs

    def setArt(self, src):
        self.data['content']['art']['src'] = src

    def setProvider(self, name, logo):
        self.data['content']['provider']['name'] = name;
        self.data['content']['provider']['logo']['src'] = logo

    def setControls(self, controls):
        self.data['controls'] = []
        if isinstance(controls, BaseButton):
            self.data['controls'].append(controls.getData())

        if type(controls) == list:
            for control in controls:
                self.data['controls'].append(control.getData())

    def addControl(self, control):
        if isinstance(control, BaseButton):
            self.data['controls'].append(control.getData())

    def getData(self):
        return self.data
    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""

from enum import Enum, unique

@unique
class RepeatButtonEnum(Enum):
    REPEAT_ONE = 'REPEAT_ONE'
    REPEAT_ALL = 'REPEAT_ALL'
    REPEAT_SHUFFLE = 'SHUFFLE'

    @staticmethod
    def inEnum(repeatButton):
        return repeatButton in RepeatButtonEnum.__members__.values()

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3
"""
    desc:pass
"""

from dueros.monitor.Utils import Utils
import json

class Response:

    def __init__(self, responseData):
        if isinstance(responseData, str):
            responseData = json.loads(responseData)
        self.data = responseData

    def getOutputSpeech(self):
        if Utils.checkKeysInDict(self.data, ['response', 'outputSpeech']):
            return self.data['response']['outputSpeech']

    def getShouldEndSession(self):
        if Utils.checkKeysInDict(self.data, ['response', 'shouldEndSession']):
            return self.data['response']['shouldEndSession']
        pass


    def getSlotName(self):

        if Utils.checkKeysInDict(self.data, ['response', 'directives']):
            directive = self.data['response']['directives']
            if directive and directive[0]['slotToElicit']:
                return directive[0]['slotToElicit']
        return None


    def getReprompt(self):

        if Utils.checkKeysInDict(self.data, ['response', 'reprompt', 'outputSpeech']):
            return self.data['response']['reprompt']['outputSpeech']
        return None

if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/2

"""
    desc:pass
"""

from dueros.card.BaseCard import BaseCard

class StandardCard(BaseCard):

    def __init__(self):
        super(StandardCard, self).__init__(['title', 'content', 'image'])
        self.data['type'] = 'standard'


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/29

"""
    desc:pass
"""
from enum import Enum, unique

@unique
class StreamFormatEnum(Enum):
    STREAM_FORMAT_MP3 = 'AUDIO_MP3'
    STREAM_FORMAT_M3U8 = 'AUDIO_M3U8'
    STREAM_FORMAT_M4A = 'AUDIO_M4A'

    @staticmethod
    def inEnum(streamFormat):
        return streamFormat in StreamFormatEnum.__members__.values()

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/2/26

"""
    desc:pass
"""
import unittest

import math


import dueros.Log as log
import logging

class BaseTest(object):

    def test_sqrt(self):
        self.assertEqual(math.sqrt(4) * math.sqrt(4), 4)


if __name__ == "__main__":
    # unittest.main()
    log.init_log("./log/my_program")  # 日志保存到./log/my_program.log和./log/my_program.log.wf，按天切割，保留7天
    logging.info("Hello World!!!")



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/16

"""
    desc:pass
"""

from dueros.monitor.model.Response import Response
from dueros.monitor.model.Request import Request
from dueros.monitor.BotMonitor import BotMonitor
from dueros.Certificate import Certificate
import json
import os
import hashlib
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA
from base64 import b64encode, b64decode
class test_botmonitor:
    pass


if __name__ == '__main__':

    privatePemContent = '''-----BEGIN RSA PRIVATE KEY-----
        MIICXQIBAAKBgQDKoeRzRVf8WoRSDYYqUzThpYCr90jfdFwTSXHJ526K8C6TEwdT
        UA+CFPQPRUg9jrYgFcown+J2myzO8BRLynD+XHb9ilLb49Mqk2CvDt/yK32lgHv3
        QVx14Dpb6h8isjncSF965fxBxlHGbvPwnHkJ9etRIYdYV3QpYohFszH3wQIDAQAB
        AoGAFhKqkw/ztK6biWClw8iKkyX3LURjsMu5F/TBK3BFb2cYe7bv7lhjSBVGPL+c
        TfBU0IvvGXrhLXBb4jLu0w67Xhggwwfc86vlZ8eLcrmYVat7N6amiBmYsw20GViU
        UFmePbo1G2BXqMA43JxqbIQwOLZ03zdw6GHj6EVlx369IAECQQD4K2R3K8ah50Yz
        LhF7zbYPIPGbHw+crP13THiYIYkHKJWsQDr8SXoNQ96TQsInTXUAmF2gzs/AwdQg
        gjIJ/dmBAkEA0QarqdWXZYbse1XIrQgBYTdVH9fNyLs1e1sBmNxlo4QMm/Le5a5L
        XenorEjnpjw5YpEJFDS4ijUI3dSzylC+QQJARqcD6TGbUUioobWB4L9GD7SPVFxZ
        c3+EgcxRoO4bNuCFDA8VO/InP1ONMFuXLt1MbCj0ru1yFCyamc63NEUDAQJBALt7
        PjGgsKCRuj6NnOcGDSbDWIitKZhnwfqYkAApfsiBQkYGO0LLaDIeAWG2KoCB9/6e
        lAQZnYPpOcCubWyDq4ECQQCrRDf0gVjPtipnPPS/sGN8m1Ds4znDDChhRlw74MI5
        FydvHFumChPe1Dj2I/BWeG1gA4ymXV1tE9phskV3XZfq
        -----END RSA PRIVATE KEY-----'''

    pubKey = '''-----BEGIN PUBLIC KEY-----
    MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDKoeRzRVf8WoRSDYYqUzThpYCr
    90jfdFwTSXHJ526K8C6TEwdTUA+CFPQPRUg9jrYgFcown+J2myzO8BRLynD+XHb9
    ilLb49Mqk2CvDt/yK32lgHv3QVx14Dpb6h8isjncSF965fxBxlHGbvPwnHkJ9etR
    IYdYV3QpYohFszH3wQIDAQAB
    -----END PUBLIC KEY-----'''

    def requestData():
        with open("../data/intent_request_tax.json", 'r', encoding='utf-8') as load_f:
            return load_f.read()

    def responseData():
        with open("../data/test_response_tax.json", 'r', encoding='utf-8') as load_f:
            return load_f.read()

    requestData = requestData()

    responseData = responseData()

    botmonitor = BotMonitor(json.loads(requestData))

    botmonitor.setEnvironmentInfo(privatePemContent, 0)

    botmonitor.setOprationTic('aa1')
    botmonitor.setOprationToc('aa1')
    botmonitor.setOprationToc('aa2')
    botmonitor.setOprationToc('aa2')
    botmonitor.setOprationToc('aa3')
    botmonitor.setEventStart()
    botmonitor.setEventEnd()
    botmonitor.setDeviceEventStart()
    botmonitor.setDeviceEventEnd()
    botmonitor.setAppName('baidu')
    botmonitor.setAudioUrl('baidu')
    botmonitor.setDeepLink('baidu')
    botmonitor.setDeepLink('baidu')
    botmonitor.setResponseData(responseData)
    botmonitor.updateData()




    def verify(data, signature):
        key = RSA.importKey(pubKey)
        digest = SHA.new()
        digest.update(data.encode('utf-8'))
        verifier = PKCS1_v1_5.new(key)
        if verifier.verify(digest, b64decode(signature)):
            return True
        return False
    orginData = '''eyJzZXJ2aWNlRGF0YSI6IHsic2RrVHlwZSI6ICJweXRob24iLCAic2RrVmVyc2lvbiI6ICIxLjAuMCIsICJyZXF1ZXN0SWQiOiAiY2Y5ZTQ1NThjYjY2NGVkMDk5M2E0ZjNmNTY2MmVmYjdfMSIsICJxdWVyeSI6ICJcdTY3ZTVcdTRlMmFcdTdhMGUiLCAicmVhc29uIjogIlVTRVJfSU5JVElBVEVEIiwgImRldmljZUlkIjogImRldmljZUlkcmVhbCIsICJyZXF1ZXN0VHlwZSI6ICJJbnRlbnRSZXF1ZXN0IiwgInVzZXJJZCI6ICI1NTE4ODEzNyIsICJpbnRlbnROYW1lIjogImlucXVpcnkiLCAic2Vzc2lvbklkIjogInNlc3Npb25JZCIsICJsb2NhdGlvbiI6IHsiYmQwOWxsIjogeyJsb25naXR1ZGUiOiAxMTYuNDE2NTA1ODU3NjUsICJsYXRpdHVkZSI6IDM5LjkyMjU4OTgyMzI2NX0sICJ3Z3M4NCI6IHsibG9uZ2l0dWRlIjogMTE2LjQwMzg3Mzk3LCAibGF0aXR1ZGUiOiAzOS45MTQ4ODkwOH0sICJiZDA5bWMiOiB7ImxvbmdpdHVkZSI6IDEyOTU5NTY3LjQwMzAzNCwgImxhdGl0dWRlIjogNDgyNzAyMS44MjM1MDc1fX0sICJzbG90VG9FbGljaXQiOiAibW9udGhseXNhbGFyeSIsICJzaG91bGRFbmRTZXNzaW9uIjogZmFsc2UsICJvdXRwdXRTcGVlY2giOiB7InR5cGUiOiAiU1NNTCIsICJ0ZXh0IjogIiIsICJzc21sIjogImphdmEtc2RrXHU2MGE4XHU3Njg0XHU3YTBlXHU1MjRkXHU1ZGU1XHU4ZDQ0XHU2NjJmXHU1OTFhXHU1YzExXHU1NDYyPyJ9LCAicmVwcm9tcHQiOiB7InR5cGUiOiAiU1NNTCIsICJ0ZXh0IjogIiIsICJzc21sIjogImphdmEtc2RrXHU2MGE4XHU3Njg0XHU3YTBlXHU1MjRkXHU1ZGU1XHU4ZDQ0XHU2NjJmXHU1OTFhXHU1YzExXHU1NDYyPyJ9LCAiYXVkaW9VcmwiOiAiYmFpZHUiLCAiYXBwSW5mbyI6IHsiYXBwTmFtZSI6ICJiYWlkdSIsICJwYWNrYWdlTmFtZSI6IG51bGwsICJkZWVwTGluayI6ICJiYWlkdSJ9LCAicmVxdWVzdFN0YXJ0VGltZSI6IDE1MTYxMDk4NDQsICJyZXF1ZXN0RW5kVGltZSI6IDE1MTYxMDk4NDQsICJ0aW1lc3RhbXAiOiAxNTE2MTA5ODQ0LCAic3lzRXZlbnQiOiB7InByZUV2ZW50TGlzdCI6IHt9LCAicG9zdEV2ZW50TGlzdCI6IHt9LCAiZXZlbnRDb3N0VGltZSI6IDAsICJkZXZpY2VFdmVudENvc3RUaW1lIjogMH0sICJ1c2VyRXZlbnQiOiB7ImFhMSI6IDAsICJhYTIiOiAwLCAiYWEzIjogMH19fQ==97d6bf29-4482-934e-c44c-626b151b46ba1516109844debug'''
    signData = '''VNt+71reYkBFvPlqMwpRmUQFXDlBSeLEyOiEu9+Sd1azfvyms/lHu9TFFHmVf6NFJA1e3f1fNhJq0XhFGvoc/gina7HTdHh1gGZrQcO/UZgQp80j6kaJAZXenQiwCz10rI/cooHQXMsh7vyVdmS14jUt+9I/Cwn6BHfxSwhFlYY='''
    print(verify(orginData,signData))
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from enum import Enum, unique


@unique
class TextContentPosition(Enum):
    TOP_LEFT = 'TOP_LEFT'
    CENTER = 'CENTER'
    BOTTOM_LEFT = 'BOTTOM_LEFT'

    @staticmethod
    def inEnum(position):

        return position in TextContentPosition.__members__.values()

if __name__ == '__main__':

    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from enum import Enum, unique


@unique
class TextType(Enum):
    PLAIN_TEXT = 'PLAIN_TEXT'
    RICH_TEXT = 'RICH_TEXT'

    @staticmethod
    def inEnum(position):
        return position in TextType.__members__.values()

if __name__ == '__main__':

    # position = TextContentPosition()
    print(TextType.PLAIN_TEXT.value)
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""

from dueros.directive.Display.template.TextStructure import TextStructure
from dueros.directive.Display.template.ImageStructure import ImageStructure
from dueros.directive.Display.template.TextType import TextType

class BaseTemplate:

    def __init__(self, field):
        super(BaseTemplate, self).__init__()
        self.data = {}
        self.supportSetField = field

    def setBackGroundImage(self, url, widthPixels='', heightPixels = ''):
        '''

        :param url:
        :param widthPixels:
        :param heightPixels:
        :return:
        '''
        if url:
            image = self.createImageStructure(url, widthPixels, heightPixels)
            if image:
                self.data['backgroundImage'] = image.getData()

    def createImageStructure(self, url, widthPixels, heightPixels):
        '''

        :param url:
        :param widthPixels:
        :param heightPixels:
        :return:
        '''
        if url:
            image = ImageStructure()
            image.setUrl(url)
            if widthPixels:
                image.setWidthPixels(widthPixels)

            if heightPixels:
                image.setHeightPixels(heightPixels)
            return image

    def createTextStructure(self, content, type=TextType.PLAIN_TEXT):
        '''

        :param content:
        :param type:
        :return:
        '''
        if content:
            textStructure = TextStructure()
            textStructure.setText(content)
            if TextType.inEnum(type):
                textStructure.setType(type.value)
            else:
                textStructure.setType(TextType.PLAIN_TEXT)

            return textStructure

    def getData(self):

        return self.data

    def __getattr__(self, item):
        '''
        添加魔术方法
        :param item:
        :return:
        '''
        # 获取操作类型 set
        operation = item[0:3]
        # 获取被操作的属性
        field = item[3:]
        if (operation == 'set' and field and (field.lower() in self.supportSetField)):
            def function(*args):
                self.data[field.lower()] = args[0]

            return function
        else:
            def function(*args):
                print('不支持', operation, field)

            return function

if __name__ == '__main__':

    bodytemplate = BaseTemplate(['token'])
    bodytemplate.setToken('a')
    bodytemplate.setBackGroundImage('aaaa')
    print(bodytemplate.getData())
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/5

"""
    desc:pass
"""


class Utils:

    @staticmethod
    def checkKeyInDict(dicts, key):
        '''
        校验字典中是否存在指定的key
        :param dicts:
        :param key:
        :return:
        '''
        if isinstance(dicts, dict):
            return key in dicts
        return False

    @staticmethod
    def checkKeysInDict(dicts, keys):

        for key in keys:
            if key in dicts:
                dicts = dicts[key]
                continue
            return False


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from enum import Enum, unique


@unique
class ThumbsUpDownButtonEnum(Enum):
    THUMBS_UP = 'THUMBS_UP'
    THUMBS_DOWN = 'THUMBS_DOWN'

    @staticmethod
    def inEnum(thumbsUpDown):
        return thumbsUpDown in ThumbsUpDownButtonEnum.__members__.values()

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- coding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/31

from dueros.card.BaseCard import BaseCard


class ImageCard(BaseCard):

    def __init__(self):
        super(ImageCard, self).__init__()
        self.data['type'] = 'image'

    def addItem(self, src, thumbnail = ''):
        '''
        添加
        :param src:
        :param thumbnail:
        :return:
        '''

        if not src:
            return self

        if not 'list' in self.data:
            self.data['list'] = []

        item = {}
        item['src'] = src

        if thumbnail:
            item['thumbnail'] = thumbnail
        self.data['list'].append(item)
        return self


if __name__ == '__main__':

    list = []

    dit = {'name':'adfa'}
    list.append(dit)

    print(list)

    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from dueros.directive.AudioPlayer.Control.BaseButton import BaseButton

class RadioButton(BaseButton):

    def __init__(self,name, selectedValue = ''):
        super(RadioButton, 'RADIO_BUTTON', name)
        self.setSelectedValue(selectedValue)

    def setSelectedValue(self, selectedValue):
        self.data['selectedValue'] = selectedValue;

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""

from dueros.monitor.Utils import Utils

class Request:

    def __init__(self, data):
        self.data = data
        self.requestType = data['request']['type']

    def getData(self):
        '''
        返回request请求头
        :return:
        '''
        return self.data

    def getDeviceData(self):
        '''

        :return:
        '''
        return self.deviceData

    def getDeviceId(self):

        if Utils.checkKeysInDict(self.data,['context','System','device','deviceId']):
            return self.data['context']['System']['device']['deviceId']
        return None

    def getAudioPlayerContext(self):
        '''
        获取设备音频播放的状态
        :return:
        '''
        if Utils.checkKeysInDict(self.data,['context','AudioPlayer']):
            return self.data['context']['AudioPlayer']
        return None

    def getAppLauncherContext(self):
        '''
        获取设备app安装列表
        :return:
        '''
        if Utils.checkKeysInDict(self.data,['context','AppLauncher']):
            return self.data['context']['AppLauncher']
        return None

    def getEventData(self):
        '''
        获取event请求
        :return:
        '''
        if self.requestType == 'IntentRequest' or self.isLaunchRequest():
            return None
        else:
            return self.data['request']

    def getUserInfo(self):
        '''

        :return:
        '''
        if Utils.checkKeysInDict(self.data, ['user_info']):
            return self.data['user_info']
        return None

    def getType(self):
        '''
        获取request类型
        :return:
        '''
        return self.requestType

    def getUserId(self):

        if Utils.checkKeysInDict(self.data, ['context', 'System', 'user', 'userId']):
            return self.data['context']['System']['user']['userId']
        return None

    def getCuid(self):

        return self.data['cuid']

    def getQuery(self):
        '''
        获取query
        :return:
        '''
        if self.requestType == 'IntentRequest':
            return self.data['request']['query']['original']
        return None

    def getLocation(self):
        if Utils.checkKeysInDict(self.data, ['context','System','user', 'userInfo','location','geo']):
            return self.data['context']['System']['user']['userInfo']['location']['geo']
        return None

    def getTimestamp(self):

        if Utils.checkKeysInDict(self.data, ['request','timestamp']):
            return self.data['request']['timestamp']
        return None

    def getLogId(self):

        return self.data['log_id']

    def getBotId(self):
        if Utils.checkKeysInDict(self.data,['context','System','application','applicationId']):
            return self.data['context']['System']['application']['applicationId']
        return None

    def getRequestId(self):

        if Utils.checkKeysInDict(self.data,['request','requestId']):
            return self.data['request']['requestId']

        return None

    def getReson(self):

        if Utils.checkKeysInDict(self.data, ['request','reason']):
            return self.data['request']['reason']
        return None

    def getIntentName(self):

        if self.data['request'] and 'intents' in self.data['request'] and self.data['request']['intents'] and self.data['request']['intents'][0]:
            return self.data['request']['intents'][0]['name']

        return None


    def getSessionId(self):

        if Utils.checkKeysInDict(self.data,['session', 'sessionId']):
            return self.data['session']['sessionId']
        return None

    def isDialogStateCompleted(self):
        if Utils.checkKeysInDict(self.data, ['request', 'dialogState']):
            return self.data['request']['dialogState'] == 'COMPLETED'
        return False

if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/16

"""
    desc:pass
"""

from dueros.monitor.model.Response import Response
import os
import json

class test_response:
    pass


if __name__ == '__main__':

    def responseData():
        with open("../data/test_response_tax.json", 'r', encoding='utf-8') as load_f:
            return load_f.read()
    responseData = responseData()
    # print(responseData)

    response = Response(json.loads(responseData))

    print(response.getSlotName())
    print(response.getShouldEndSession())
    print(response.getOutputSpeech())
    print(response.getReprompt())

    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/31

"""
    用于生成Play指令的类
"""

from dueros.directive.BaseDirective import BaseDirective
from dueros.directive.AudioPlayer.PlayBehaviorEnum import PlayBehaviorEnum
from dueros.directive.AudioPlayer.StreamFormatEnum import StreamFormatEnum
from dueros.directive.AudioPlayer.PlayerInfo import PlayerInfo
from dueros.directive.AudioPlayer.Control.PlayPauseButton import PlayPauseButton
from dueros.directive.AudioPlayer.Control.PreviousButton import PreviousButton
from dueros.directive.AudioPlayer.Control.NextButoon import NextButoon

class Play(BaseDirective):

    def __init__(self, url, playBehavior = PlayBehaviorEnum.REPLACE_ALL):
        '''

        :param url:     音频播放地址
        :param playBehavior: REPLACE_ALL: 立即停止当前播放并清除播放队列，立即播放指令中的audio item。
                             ENQUEUE: 将audio item添加到当前队列的尾部。
                             REPLACE_ENQUEUED: 替换播放队列中的所有audio item，但不影响当前正在播放的audio item。
        '''

        super(Play, self).__init__('AudioPlayer.Play')
        self.data['playBehavior'] = playBehavior.value
        self.data['audioItem'] = {
            'stream': {
                'streamFormat': StreamFormatEnum.STREAM_FORMAT_MP3.value,
                'url': url,
                'offsetInMilliSeconds': 0,
                'token': self.genToken()
            }
        }

    def setPlayerInfo(self, playerInfo):
        if isinstance(playerInfo,PlayerInfo):
            self.data['audioItem']['playerInfo'] = playerInfo.getData()

    def setToken(self, token):
        if token:
            self.data['audioItem']['stream']['token'] = token
        return self

    def getToken(self):
        return self.data['audioItem']['stream']['token']

    def setUrl(self, url):
        if url:
            self.data['audioItem']['stream']['url'] = url
        return self

    def setOffsetInMilliSeconds(self, milliSeconds):
        '''
        设置directive的属性。从指定的offset开始进行播放
        :param milliSeconds:    毫秒数。比如5分钟的歌曲，播放的长度是5*60*1000毫秒，选择起始的播放位置
        :return:
        '''
        if milliSeconds.isdigit():
            milliSeconds = int(milliSeconds)
            self.data['audioItem']['stream']['offsetInMilliSeconds'] = milliSeconds
        return self

    def setProgressReportIntervalMs(self, intervalMs):
        '''
        设置directive的属性。定时上报事件的间隔时间
        :param intervalMs:  毫秒数。
        :return:
        '''
        if intervalMs.isdigit():
            intervalMs = int(intervalMs)
            self.data['audioItem']['stream']['progressReportIntervalMs'] = intervalMs
        return self

    def setStreamFormat(self, streamFormat=StreamFormatEnum.STREAM_FORMAT_MP3):
        '''
        设置directive的属性。音频流格式，streamFormat 默认STREAM_FORMAT_MP3
        :param streamFormat:    取值: STREAM_FORMAT_MP3、STREAM_FORMAT_M3U8、STREAM_FORMAT_M4A
        :return:
        '''
        if StreamFormatEnum.inEnum(streamFormat):
            self.data['audioItem']['stream']['streamFormat'] = streamFormat.value
        else:
            self.data['audioItem']['stream']['streamFormat'] = StreamFormatEnum.STREAM_FORMAT_MP3.value
        return self


if __name__ == '__main__':
    directive = Play('http://www.baidu.com')
    directive.setStreamFormat('AUDIO_M3U8')

    playerInfo = PlayerInfo()

    # 创建暂停按钮
    playpause = PlayPauseButton()
    previous = PreviousButton()
    previous.setSelected(True)
    controls = [playpause, previous]
    playerInfo.setControls(controls)
    playerInfo.addControl(NextButoon())

    playerInfo.setTitle('周杰伦')
    playerInfo.setTitleSubtext1('七里香')
    playerInfo.setArt('http://adfadfa')

    # 设置Play指令的PlayerInfo
    directive.setPlayerInfo(playerInfo)
    print(directive.getData())




#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/16

"""
    desc:pass
"""
from dueros.monitor.model.Request import Request
import os
import json

class test_request:
    pass


if __name__ == '__main__':

    def requestData():
        with open("../data/intent_request_tax.json", 'r', encoding='utf-8') as load_f:
            return load_f.read()
    requestData = requestData()

    request = Request(json.loads(requestData))

    print(request.getType())
    print(request.getUserId())
    print(request.getQuery())
    print(request.getBotId())
    print(request.getRequestId())
    print(request.getReson())
    print(request.getIntentName())
    print(request.getSessionId())
    print(request.getDeviceId())
    print(request.getLocation())
    print(request.isDialogStateCompleted())
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""

from dueros.Bot import Bot
from dueros.card.TextCard import TextCard
import json

class Bot(Bot):
    def launchRequest(self):
        return {
            'card': TextCard(r'欢迎使用家居控制!请告诉我您要查找什么智能设备，比如查找我的空调'),
            'outputSpeech': r'<speak>欢迎使用家居控制!请告诉我您要查找什么智能设备，比如查找我的空调</speak>'
        }

    def intentRequest(self):
        self.ask('deviceName')

        card = TextCard('您要查找什么智能设备呢? 比如"查找我的空调"')
        card.addCueWords("百度")
        card.addCueWords("百度")
        card.addCueWords("百度")
        card.setAnchor("http://www.baidu.com", "百度")
        return {
            'card': card,
            'outputSpeech': '<speak>您要查找什么智能设备呢? 比如"查找我的空调"</speak>'
        }

    def controlRequest(self):
        self.ask('deviceName')
        # deviceName = self.getSlots('deviceName')
        # print('deviceName %s' % (deviceName))
        return {
            'card': TextCard('请告诉您的指令，比如调小空调风速、设置温度为30度'),
            'outputSpeech': '请告诉您的指令，比如调小空调风速、设置温度为30度'
        }

    def __init__(self, data):
        super(Bot, self).__init__(data)

        self.addLaunchHandler(self.launchRequest)

        self.addIntentHandler('dueros.device_interface.smart_device.control', self.controlRequest)

        self.addIntentHandler('dueros.device_interface.smart_device.search', self.intentRequest)
    pass


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from dueros.directive.AudioPlayer.Control.RadioButton import RadioButton
from dueros.directive.AudioPlayer.Control.RepeatButtonEnum import RepeatButtonEnum

class RepeatButton(RadioButton):

    def __init__(self, selectedValue = RepeatButtonEnum.REPEAT_ONE.value):
        super(RepeatButton, self).__init__('REPEAT', selectedValue)

    def setSelectedValue(self, selectedValue = RepeatButtonEnum.REPEAT_ONE):

        if RepeatButtonEnum.inEnum(selectedValue):
            self.data['selectedValue'] = selectedValue.value
        else:
            self.data['selectedValue'] = RepeatButtonEnum.REPEAT_ONE


    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/29

"""
    desc:pass
"""
from enum import Enum, unique

@unique
class PlayBehaviorEnum(Enum):
    REPLACE_ALL = 'REPLACE_ALL'
    REPLACE_ENQUEUED = 'REPLACE_ENQUEUED'
    ENQUEUE = 'ENQUEUE'


    @staticmethod
    def inEnum(playBehavior):
        return playBehavior in PlayBehaviorEnum.__members__.values()

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/27

"""
    desc:pass
"""

from dueros.directive.BaseDirective import BaseDirective
from dueros.directive.Display.template.BaseTemplate import BaseTemplate

class RenderTemplate(BaseDirective):

    def __init__(self):
        super(RenderTemplate, self).__init__('Display.RenderTemplate')

    def __init__(self, baseTemplate):
        super(RenderTemplate, self).__init__('Display.RenderTemplate')
        self.setTemplate(baseTemplate)

    def setTemplate(self, template):
        '''
        设置模板
        :param template:
        :return:
        '''
        if isinstance(template, BaseTemplate):
            self.data['template'] = template.getData()


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/2

from dueros.directive.BaseDirective import BaseDirective
import logging

class LaunchApp(BaseDirective):
    """
       用于调用app的指令类
    """

    def __init__(self, appName='', packageName='', deepLink=''):
        '''
        三者必须传一个
        :param appName:     应用名称
        :param packageName: 应用包
        :param deepLink:    打开应用指定功能
        '''

        super(LaunchApp, self).__init__('AppLauncher.LaunchApp')
        if not appName and not packageName and not deepLink:
            print('appName packageName deepLink 必须要有一个')
        else:
            self.data = dict({
                'appName': appName,
                'packageName': packageName,
                'deepLink': deepLink,
                'token': self.genToken()
            },**self.data)

    def setAppName(self, appName):

        if appName:
            self.data['appName'] = appName
        return self

    def setPackageName(self, packageName):

        if packageName:
            self.data['packageName'] = packageName
        return self

    def setDeepLink(self, deepLink):

        if deepLink:
            self.data['deepLink'] = deepLink
        return self

if __name__ == '__main__':

    launchApp = LaunchApp('', '', '2')
    launchApp.setDeepLink('dd')
    print(launchApp.data)




#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/31

"""
    desc:pass
"""
from dueros.directive.BaseDirective import BaseDirective

class Hint(BaseDirective):

    def __init__(self, text):
        super(Hint, self).__init__('Hint')
        self.data['hints'] = []
        if type(text) == str:
            text = [text]

        if type(text) == list:
            for value in text:
                item = {}
                item['type'] = 'PlainText'
                item['text'] = value
                self.data['hints'].append(item)

if __name__ == '__main__':

    hint = Hint('aaa')
    print(hint.getData())
    pass


#!/usr/bin/env python3
# -*- coding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/30


class BaseCard(object):
	'''
	卡片基类
	'''

	def __init__(self, field = []):
		self.data = {}
		self.supportSetField = field

	def addCueWords(self, arr):
		'''
		为卡片添加cue words 提示用户输入
		:param arr: 数组
		:return:
		'''
		if arr:
			if isinstance(arr, str):
				arr = [arr]

			if 'cueWords' in self.data:
				self.data['cueWords'] = self.data['cueWords']
			else:
				self.data['cueWords'] = []

			self.data['cueWords'].extend(arr)
		return self

	def setAnchor(self, url, anchorText):
		'''
		设置卡片链接
		:param url:	 比如:http(s)://....
		:param anchorText:	链接显示的文字
		:return:
		'''
		if url:
			self.data['url'] = url
			if(anchorText):
				self.data['anchorText'] = anchorText
		return self

	def getData(self):
		return self.data

	def __getattr__(self, item):
		'''
		添加魔术方法
		:param item:
		:return:
		'''
		#获取操作类型 set
		operation = item[0:3]
		#获取被操作的属性
		field = item[3:]
		if(operation == 'set' and field and (field.lower() in self.supportSetField)):
			def function(*args):
				self.data[field.lower()] = args[0]
			return function
		else:
			def function(*args):
				print('不支持', operation, field)
			return function

if __name__ == '__main__':

	pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""

from dueros.directive.Display.template.BaseTemplate import BaseTemplate
from dueros.directive.Display.template.TextType import TextType

class ListTemplateItem(BaseTemplate):

    def __init__(self):
        super(ListTemplateItem, self).__init__(['token', 'image'])
        self.data['textContent'] = {}

    def setPlainPrimaryText(self, primaryText):
        '''
        设置一级标题
        :param primaryText:
        :return:
        '''
        primaryTextStructure = self.createTextStructure(primaryText, TextType.PLAIN_TEXT)
        if primaryTextStructure:
           self.data['textContent']['primaryText'] = primaryTextStructure.getData()

    def setPlainSecondaryText(self, secondaryText):
        '''
        设置二级标题
        :param secondaryText:
        :return:
        '''
        secondaryTextStructure = self.createTextStructure(secondaryText, TextType.PLAIN_TEXT)
        if secondaryTextStructure:
            self.data['textContent']['secondaryText'] = secondaryTextStructure.getData()
        pass

    def setTertiaryText(self, tertiaryText):
        '''
        设置三级标题
        :param tertiaryText:
        :return:
        '''
        tertiaryTextStructure = self.createTextStructure(tertiaryText, TextType.PLAIN_TEXT)
        if tertiaryTextStructure:
            self.data['textContent']['tertiaryText'] = tertiaryTextStructure.getData()
        pass

    def setImage(self, url, widthPixels='', heightPixels = ''):
        '''
        设置
        :param url:
        :param widthPixels:
        :param heightPixels:
        :return:
        '''
        image = self.createImageStructure(url, widthPixels, heightPixels)
        if image:
            self.data['image'] = image.getData()

if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/30

import json
from dueros.Nlu import Nlu
from dueros.Session import Session
from dueros.Base import Base

class Request(Base):
    '''
        DuerOS对Bot的请求封装
    '''

    def __init__(self, data):
        '''

        :param data:  请求数据
        '''
        super(Request, self).__init__()

        self.data = json.loads(data)
        self.requestType = self.data['request']['type']
        self.session = Session(self.data['session'])
        self.nlu = None
        if self.requestType == 'IntentRequest':
            self.nlu = Nlu(self.data['request']['intents'])
        self.deviceData = None
        self.arrUserProfile = None


    def getData(self):
        '''
        返回request 请求体
        :return:
        '''
        return self.data

    def getSession(self):
        '''
        返回Session实例
        :return:
        '''
        return self.session

    def getNlu(self):
        '''
        获取nlu实例
        :return:
        '''
        return self.nlu

    def getDeviceData(self):
        '''
        返回设备信息
        :return:
        '''
        return self.deviceData

    def getDeviceId(self):
        '''
        获取设备Id
        :return:
        '''

        return self.data['context']['System']['device']['deviceId']

    def getOriginalDeviceId(self):
        '''
        获取来自端上报的原始设备Id
        :return:
        '''
        return self.data['context']['System']['device']['originalDeviceId']

    def getAudioPlayerContext(self):
        '''
        获取设备音频播放状态
        :return:
        '''
        return self.data['context']['AudioPlayer']

    def getVideoPlayerContext(self):

        return self.data['context']['VideoPlayer']

    def getScreenContext(self):
        return self.data['context']['Screen']

    def getScreenTokenFromContext(self):
        return self.data['context']['Screen']['token']

    def getScreenTokenFromContext(self):
        return self.data['context']['Screen']['token']
    def getScreenCardFromContext(self):
        return self.data['context']['Screen']['card']


    def getAppLauncherContext(self):
        '''
        获取设备app安装列表
        :return:
        '''

        return self.data['context']['AppLauncher']

    def getEventData(self):
        '''
        获取event请求
        :return:
        '''

        if self.requestType == 'IntentRequest' or self.isSessionEndedRequest() or self.isLaunchRequest():
            return
        else:
            return self.data['request']

    def getUserInfo(self):
        '''
        获取用户信息
        :return:
        '''

        return self.data['context']['System']['user']['userInfo']

    def getBaiduUid(self):
        '''
        获取百度Id
        :return:
        '''

        return self.data['context']['System']['user']['userInfo']['account']['baidu']['baiduUid']

    def getType(self):
        '''
        获取Request类型
        :return:
        '''
        return self.requestType

    def getUserId(self):
        '''
        获取用户ID
        :return:
        '''
        return self.data['context']['System']['user']['userId']

    def getAccessToken(self):

        '''
        获取accessToken
        :return:
        '''
        return self.__getSystemUser()['accessToken']

    def __getSystemUser(self):

        return self.data['context']['System']['user']

    def getExternalAccessTokens(self):
        '''
        获取
        :return:
        '''

        return self.__getSystemUser()['externalAccessTokens']

    def getCuid(self):
        return self.data['cuid']

    def getQuery(self):
        '''
        获取请求的Query
        :return:
        '''

        if self.requestType == 'IntentRequest' and self.data['request']['query']['original']:
            return self.data['request']['query']['original']
        else:
            return ''

    def getLocation(self):
        '''
        获取设备位置信息
        :return:
        '''
        if self.__getSystemUser()['userInfo']['location']:
            return self.__getSystemUser()['userInfo']['location']

    def isDetermined(self):

        if self.requestType == 'IntentRequest' and self.data['request']['determined']:
            return self.data['request']['determined']
        else:
            return False

    def isLaunchRequest(self):
        '''
        是否为调起bot请求
        :return:
        '''
        return self.data['request']['type'] == 'LaunchRequest'

    def isSessionEndRequest(self):
        '''
        是否关闭bot请求
        :return:
        '''
        return self.data['request']['type'] == 'SessionEndedRequest'

    def isSessionEndedRequest(self):
        return self.isSessionEndRequest()

    def getTimestamp(self):

        if self.data['request']['timestamp']:
            return self.data['request']['timestamp']

    def getLogId(self):

        if self.data['request']['requestId']:
            return self.data['request']['requestId']

    def getBotId(self):

        if self.data['context']['System']['application']['applicationId']:
            return self.data['context']['System']['application']['applicationId']

    def isDialogStateCompleted(self):
        '''
        槽位是否填完

        :return:
        '''
        return self.data['request']['dialogState'] == 'COMPLETED'



if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/31 上午12:25

"""
    用于生成Stop指令的类
"""
from dueros.directive.BaseDirective import BaseDirective


class Stop(BaseDirective):

    def __init__(self):
        super(Stop, self).__init__('AudioPlayer.Stop')


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/30

import json
import re
import logging

from dueros.monitor.BotMonitor import BotMonitor
from dueros.Certificate import Certificate
from dueros.Intercept import Intercept
from dueros.Request import Request
from dueros.Response import Response
from dueros.Base import Base

class Bot(Base):
    '''
    Bot入口
    '''

    def __init__(self, postdata):
        '''
        构造方法
        :param postData:
        '''
        super(Bot, self).__init__()
        self.postData = postdata
        self.request = Request(postdata)
        self.session = self.request.getSession()
        self.nlu = self.request.getNlu()
        self.response = Response(self.request, self.session, self.nlu)
        self.handler = []
        # self.botMonitor = BotMonitor(postdata)
        self.intercept = []
        self.certificate = None
        self.callBackFunc = None
        self.cakkBackData = None
        self.event = {}
        logging.info('Bot init')

    def initCertificate(self, environ):
        '''
        创建Certificate
        :param environ:
        :return:
        '''

        self.certificate = Certificate(environ, self.postData)
        return self

    def enableVerifyRequestSign(self):
        '''
        开启签名验证
        :return:
        '''

        if self.certificate:
            self.certificate.enableVerifyRequestSign()
        return self

    def disableVerifyRequestSign(self):
        '''
        关闭签名验证
        :return:
        '''

        if self.certificate:
            self.certificate.disableVerifyRequestSign()
        return self

    def addLaunchHandler(self, func):
        '''
        添加对LaunchRequest的处理函数
        :param func:    回调方法
        :return:
        '''

        return self.__addHandler('LaunchRequest', func)

    def addSessionEndedHandler(self, func):
        '''
        添加对SessionEndedRequest的处理函数
        :param func:    回调方法
        :return:
        '''

        return self.__addHandler('SessionEndedRequest', func)

    def addIntentHandler(self, intentName, func):
        '''
        添加对特定意图的处理函数
        :param intentName:  意图英文标识名
        :param func:    回调方法
        :return:
        '''

        return self.__addHandler('#' + intentName, func)

    def __addHandler(self, mix, func):
        '''
        私有方法
        添加Handler，条件处理顺序相关，优先匹配先添加的条件
            1、如果满足，则执行，有返回值则停止
            2、满足条件，执行返回null,继续寻找下一个满足的条件
        :param mix:     条件，比如意图以'#'开头的'#intentName'或者'LaunchRequest'、'SessionEndedRequest'
        :param func:    处理函数，满足mix条件后执行该函数
        :return:
        '''

        if isinstance(mix, str) and hasattr(func, '__call__'):
            arr = {mix: func}
            mix = arr

        if not isinstance(mix, dict):
            return

        for key, value in mix.items():

            if not key or not value:
                return

            self.handler.append({
                'rule': key,
                'func': value
            })
        return self

    def addIntercept(self, intercept):
        '''
        添加拦截器
        :param intercept:
        :return:
        '''

        if isinstance(intercept, Intercept):
            self.intercept.append(intercept)

    def addEventListener(self, event, func):
        '''
        绑定一个事件的处理回调
        @link http://developer.dueros.baidu.com/doc/dueros-conversational-service/device-interface/audio-player_markdown 具体事件参考

        @example
        <pre>
        self.addEventListener('AudioPlayer.PlaybackStarted', function(event){
          return {
              'outputSpeech' => '事件处理好啦'
          }
        })
        </pre>
        :param event:   绑定的事件名称，比如AudioPlayer.PlaybackStarted
        :param func:    处理函数，传入参数为事件的request，返回值做完response给DuerOS
        :return:
        '''

        if event and func:
            self.event[event] = func

    def addDefaultEventListener(self, func):
        '''
        默认兜底事件的处理函数
        :param event:
        :param func:
        :return:
        '''
        if hasattr(func, '__call__'):
            self.event['__default__'] = func

    def getIntentName(self):
        '''
        获取第一个Intent的名字
        :return:
        '''

        if self.nlu:
            return self.nlu.getIntentName()

    def getSessionAttribute(self, field, default):
        '''
        获取session某个字段值
        :param field:
        :param default:
        :return:
        '''

        return self.session.getData(field, default)

    def setSessionAttribute(self, field, value, default):
        '''
        设置session某个字段值
        :param field:
        :param value:
        :param default:
        :return:
        '''

        self.session.setData(field, value, default)

    def clearSessionAttribute(self):
        '''
        清空session
        :return:
        '''

        self.session.clear()

    def getSlots(self, field, index = 0):
        '''
        获取槽位值
        :param field:
        :param index:
        :return:
        '''

        if self.nlu:
            return self.nlu.getSlot(field, index)

    def setSlots(self, field, value, index = 0):
        '''
        设置槽位值
        :param field:
        :param value:
        :param index:
        :return:
        '''

        if self.nlu:
            self.nlu.setSlot(field, value, index)

    def waitAnswer(self):
        '''
        告诉DuerOS, 在多轮对话中，等待用户回答
        :return:
        '''

        if self.response:
            self.response.setShouldEndSession(False)

    def endDialog(self):
        '''
        告诉DuerOS 需要结束对话
        :return:
        '''

        if self.response:
            self.response.setShouldEndSession(True)

    def endSession(self):
        '''
        告诉DuerOS 需要结束对话
        :return:
        '''
        self.endDialog()

    def run(self, build = True):
        '''
        事件路由添加后，需要执行此函数，对添加的条件、事件进行判断
        将第一个return 非null的结果作为此次的response
        :param build: False:不进行response封装，直接返回handler的result
        :return:
        '''

        if self.certificate and not self.certificate.verifyRequest():
            return self.response.illegalRequest()

        eventHandler = self.__getRegisterEventHandler()

        if self.request.getType() == 'IntentRequest' and not self.nlu and not eventHandler:
            return self.response.defaultResult()

        ret = {}
        # for intercept in self.intercept:
            # self.botMonitor.setPreEventStart()
            # ret = intercept.preprocess(self)
            # self.botMonitor.setPreEventEnd()
            # if(ret):
            #     return

        if eventHandler:
            # self.botMonitor.setDeviceEventStart()
            event = self.request.getEventData()
            ret = self.__callFunc(eventHandler, event)
            # self.botMonitor.setDeviceEventEnd()
        else:
            # self.botMonitor.setEventStart()
            ret = self.__dispatch()
            # self.botMonitor.setEventEnd()

        # for intercept in self.intercept:
            # self.botMonitor.setPostEventStart()
            # ret = intercept.postprocess(self, ret)
            # self.botMonitor.setPostEventEnd()

        if not build:
            if self.cakkBackData:
                return json.dumps(self.cakkBackData)
            else:
                return json.dumps(ret)
        if not ret:
            ret = {}
        res = self.response.build(ret)
        print(json.dumps(res))
        if self.cakkBackData:
            return json.dumps(self.cakkBackData)
        else:
            return json.dumps(res)
    
    def __dispatch(self):
        '''
        分发请求并调用回调方法
        :return:
        '''

        if not self.handler:
            return

        #循环遍历handler 通过正则判断调用哪个handler
        for item in self.handler:
            if item:
                #获取rule(其实是自己的技能意图的英文标识)
                rule = item['rule']
                #校验handler
                if self.__checkHandler(rule):
                    #匹配到handler获取对应的回调方法并立即执行
                    func = item['func']
                    ret = self.__callFunc(func, None)
                    if ret:
                        return ret
        #调用回调
        self.unMatchHandler(self.cakkBackData)


    def __getRegisterEventHandler(self):

        eventData = self.request.getEventData()
        if eventData and eventData['type']:
            key = eventData['type']
            if self.event[key]:
                return self.event[key]
            elif self.event['__default__']:
                return self.event['__default__']

    def __callFunc(self, func, arg):
        '''
        自定义方法调用
        :param func:    可以为方法、字符串，如果是字符串默认调用Bot的方法
        :param arg:     参数
        :return:
        '''

        ret = ''
        if hasattr(func, '__call__'):
            if(arg == None):
                ret = func()
            else:
                ret = func(arg)
        elif isinstance(func, str):
            directive_func = getattr(self, func, None)
            if directive_func:
                ret = directive_func(arg)
        return ret

    def getToken(self, rule):
        '''

        :param rule:
        :return:
        '''

        token = {}
        return self.getSlots(token, rule)
        pass

    def __getToke(self, token, rule):
        '''

        :param token:
        :param rule:
        :return:
        '''

        if rule == '' or not rule:
            return token
        pass

    def __checkHandler(self, handler):
        '''
        根据意图标识英文名 和 请求类型判断是否是此handler
        :param handler:
        :return:
        '''

        rg = {
            'intent': r'#([\w\.\d_]+)',
            'requestType': r'^(LaunchRequest|SessionEndedRequest)$'
        }

        if re.match(rg['requestType'], handler):
            if self.request.getType() == handler:
                self.cakkBackData = None
                return True
            else:
                self.unMatchHandler({'type': 'requestType', 'message': u'未匹配到:' + self.request.getType()})

        if re.match(rg['intent'], handler):
            if ('#' + self.getIntentName()) == handler:
                self.cakkBackData = None
                return True
            else:
                self.cakkBackData = {'type': 'intent', 'message': u'handler未匹配到:' + self.getIntentName()}

        if handler == 'true' or handler == True:
            return True

        return False

    def setCallBack(self, func):
        '''
        设置回调方法
        :param func:
        :return:
        '''
        if hasattr(func, '__call__'):
            self.callBackFunc = func

    def unMatchHandler(self, data):
        '''
        未匹配到Handler回调
        :param func:
        :return:
        '''
        if self.callBackFunc and data:
            self.callBackFunc(data)

    #TODO
    def tokenValue(self, str):
        '''

        :param str:
        :return:
        '''
        pass

    def declareEffect(self):
        self.response.setNeedDetermine()

    def effectConfirmed(self):
        self.request.isDetermined()

    def setExpectSpeech(self, expectSpeech):
        '''
        通过控制expectSpeech来控制麦克风开
        :param expectSpeech:
        :return:
        '''

        self.response.setExpectSpeech(expectSpeech)

    def setFallBack(self):
        '''
        标识本次返回的结果是兜底结果
        :return:
        '''

        self.response.setFallBack()

    def ask(self, slot):
        if self.nlu:
            self.nlu.ask(slot)


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from dueros.directive.AudioPlayer.Control.Button import Button

class RecommendButton(Button):

    def __init__(self):
        super(RecommendButton, self).__init__('RECOMMEND')

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""


from dueros.directive.Display.template.ListTemplate import ListTemplate
from dueros.directive.Display.template.ListTemplateItem import ListTemplateItem

class ListTemplate2(ListTemplate):

    def __init__(self):
        super(ListTemplate2, self).__init__('ListTemplate2')

if __name__ == '__main__':

    listTemplate = ListTemplate2()
    listTemplate.setToken('toke')
    listTemplate.setBackGroundImage('http://www.baidu.com')
    listTemplate.setTitle('title')

    listTemplateItem = ListTemplateItem()
    listTemplateItem.setToken('aaa')
    listTemplateItem.setImage('http://wwww.www')
    listTemplateItem.setPlainPrimaryText('一级')
    listTemplateItem.setPlainSecondaryText('二级')
    listTemplateItem.setTertiaryText('三级')
    listTemplate.addItem(listTemplateItem)
    print(listTemplate.getData())
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""


from dueros.directive.Display.template.BaseTemplate import BaseTemplate
from dueros.directive.Display.template.ListTemplateItem import ListTemplateItem

class ListTemplate(BaseTemplate):

    def __init__(self, type):
        super(ListTemplate, self).__init__(['token','title','type'])
        self.setType(type)
        self.data['listItems'] = []

    def addItem(self, item):
        if isinstance(item, ListTemplateItem):
            self.data['listItems'].append(item.getData())
        return self


if __name__ == '__main__':

    #
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3
"""
    desc:pass
"""
from dueros.monitor.model.Request import Request
from dueros.monitor.model.Response import Response
from dueros.monitor.BotMonitorConfig import BotMonitorConfig
from dueros.Certificate import Certificate
import json
import base64
import time
import requests
import threading

class BotMonitor:

    def __init__(self, postData):
        if isinstance(postData, str):
            postData = json.loads(postData)
        self.data = postData
        self.requestStartTime = self.getMillisecond()
        self.requestEndTime = 0
        self.request = Request(postData)
        self.audioUrl = None
        self.appName = None
        self.packageName = None
        self.deepLink = None
        self.eventStartTime = 0
        self.eventCostTime = 0
        self.deviceEventStartTime = 0
        self.deviceEventCostTime = 0
        self.userEventList = {}
        self.isEventMakePair = {}
        self.config = BotMonitorConfig()
        self.privateKey = None
        self.environment = 0
        self.enabled = True
        self.certificate = None
        self.response = None

    def setEnvironmentInfo(self, privateKey, environment):

        self.privateKey = privateKey
        self.environment = environment
        self.certificate = Certificate(None, self.data, privateKey)

    def setMonitorEnabled(self, enabled):
        '''
        设置是否可用
        :param enabled:
        :return:
        '''
        print('setMonitorEnable = %s' % enabled)
        self.enabled = enabled

    def setResponseData(self, responseData):

        if self.isShouldDisable():
            return
        self.requestEndTime = self.getMillisecond()
        self.response = Response(responseData)

    def setEventStart(self):

        if self.isShouldDisable():
            return
        self.eventStartTime = self.getMillisecond()

    def setEventEnd(self):

        if self.isShouldDisable():
            return
        self.eventCostTime = self.getMillisecond() - self.eventStartTime

    def setDeviceEventStart(self):

        if self.isShouldDisable():
            return
        self.deviceEventStartTime = self.getMillisecond()

    def setDeviceEventEnd(self):

        if self.isShouldDisable():
            return
        self.deviceEventCostTime = self.getMillisecond() - self.deviceEventStartTime

    def setOprationTic(self, taskName):

        if self.isShouldDisable():
            return
        if taskName:
            self.userEventList[taskName] = self.getMillisecond()
            self.isEventMakePair[taskName] = False

    def setOprationToc(self, taskName):

        if self.isShouldDisable():
            return
        if taskName:

            if taskName in self.userEventList:
                oldTime = self.userEventList[taskName]
            oldTime = None
            currTime = self.getMillisecond()
            costTime = 0

            if oldTime:
                costTime = currTime = oldTime

            self.userEventList[taskName] = costTime
            self.isEventMakePair[taskName] = True

    def setAppName(self, appName):

        if self.isShouldDisable():
            return
        if appName:
            self.appName = appName

    def setPackageName(self, packageName):

        if self.isShouldDisable():
            return
        if packageName:
            self.packageName = packageName

    def setDeepLink(self, deepLink):

        if self.isShouldDisable():
            return
        if deepLink:
            self.deepLink = deepLink

    def setAudioUrl(self, audioUrl):

        if self.isShouldDisable():
            return
        if audioUrl:
            self.audioUrl = audioUrl

    def updateData(self):
        print('botMonitor updata')
        if self.isShouldDisable():
            return
        botId = self.request.getBotId()

        #组装数据 返回元祖(base64后的data, 时间戳)
        tup = self.__buildUploadData()

        base64Data = tup[0]
        timestamp = tup[1]
        pkversion = tup[2]
        signData = "%s%s%s%s" % (base64Data, botId, timestamp, pkversion)
        print('signData = %s' % signData)
        signature = self.certificate.getSign(signData)
        print('signature = %s' % (str(signature, 'utf-8')))

        if not signature or len(pkversion) == 0:
            return

        print('content-length=%s, signature=%s, botId=%s, timestamp=%s' % (str(len(base64Data)),signature,str(botId), str(timestamp)))
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Content-Length': str(len(base64Data)),
            'SIGNATURE': signature,
            'botId': str(botId),
            'timestamp': str(timestamp),
            'pkversion': str(pkversion)
        }

        thread = threading.Thread(target=self.__uploadData, args=(base64Data, headers))
        thread.start()

    def __uploadData(self, data, headers):
        '''
        发送请求
        :param url:
        :param data:
        :param headers:
        :return:
        '''
        response = requests.post(self.config.getUploadUrl(), data=data, headers=headers)
        print(response)

    def __buildUploadData(self):

        sysEvent = {
            'preEventList': {},
            'postEventList': {},
            'eventCostTime': self.eventCostTime,
            'deviceEventCostTime': self.deviceEventCostTime
        }

        timestamp = self.getMillisecond()

        retData = {
            'serviceData': {
                'sdkType': self.config.getSdkType(),
                'sdkVersion': self.config.getSdkVersion(),
                'requestId': self.request.getRequestId(),
                'query': self.request.getQuery(),
                'reason': self.request.getReson(),
                'deviceId': self.request.getDeviceId(),
                'requestType': self.request.getType(),
                'userId': self.request.getUserId(),
                'intentName': self.request.getIntentName(),
                'sessionId': self.request.getSessionId(),
                'location': self.request.getLocation(),
                'slotToElicit': self.response.getSlotName(),
                'shouldEndSession': self.response.getShouldEndSession(),
                'outputSpeech': self.response.getOutputSpeech(),
                'reprompt': self.response.getReprompt(),
                'audioUrl': self.audioUrl,
                'appInfo': {
                    'appName': self.appName,
                    'packageName': self.packageName,
                    'deepLink': self.deepLink
                },
                'requestStartTime': self.requestStartTime,
                'requestEndTime': self.requestEndTime,
                'timestamp': timestamp,
                'sysEvent': sysEvent,
                'userEvent': self.userEventList
            }
        }

        orginData = json.dumps(retData)
        print('orginData = %s' % orginData)

        base64Data = str(base64.b64encode(orginData.encode('utf-8')), 'utf-8')
        print(base64Data)
        pkversion = None
        if self.environment == 0:
            pkversion = 'debug'
        else:
            pkversion = 'online'

        return (base64Data, timestamp, pkversion)

    def isShouldDisable(self):

        if not self.privateKey or len(self.privateKey) == 0 or not self.enabled:
            return True
        return False


    def getMillisecond(self):

        return int(time.time())

if __name__ == '__main__':

    print(int(time.time()))
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/31

"""
    desc:pass
"""
from dueros.directive.BaseDirective import BaseDirective
from dueros.directive.AudioPlayer.PlayBehaviorEnum import PlayBehaviorEnum

class VideroPlayer(BaseDirective):

    def __init__(self, url, playBehavior = PlayBehaviorEnum.REPLACE_ALL):
        super(VideroPlayer, self).__init__('VideoPlay.Play')
        self.data['playBehavior'] = playBehavior.value
        self.data['videoItem'] = {
            'videoItemId': self.getToken(),
            'stream':{
                'url': url,
                'offsetInMilliseconds': 0,
                'token': self.getToken()
            }
        }

    def setToken(self, token):
        if token:
            self.data['videoItem']['stream']['token'] = token

    def getToken(self):

        return self.data['videoItem']['stream']['token']

    def setUrl(self, url):

        if url:
            self.data['videoItem']['stream']['url'] = url

    def setOffsetInMilliseconds(self, milliseconds):

        if milliseconds.isdigit():
            milliseconds = int(milliseconds)
            self.data['videoItem']['stream']['offsetInMilliseconds'] = milliseconds

    def setExpiryTime(self, expiryTime):
        self.data['videoItem']['stream']['expiryTime'] = expiryTime

    def setReportDelayInMs(self, reportDelayMs):
        self.data['videoItem']['stream']['progressReport']['progressReportDelayInMilliseconds'] = int(reportDelayMs)

    def setReportIntervalInMs(self, intervalMs):
        self.data['videoItem']['stream']['progressReport']['progressReportIntervalInMilliseconds'] = int(intervalMs)

    def setExpectedPreviousToken(self, previousToken):
        self.data['videoItem']['stream']['expectedPreviousToken'] = previousToken


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/30

"""
"""

import time
import random
import hashlib


class BaseDirective(object):
    '''
    指令基类
    '''

    def __init__(self, type):
        self.data = {}

        self.data['type'] = type

    def genToken(self):
        '''
        生成Token md5(9位随机数+时间戳) 再截取md5后的字符串
        :return: uuid
        '''
        #生成随机数
        rand = str(random.randint(0, 9999999999))
        t = str(round(time.time() * 1000))
        md5Str = rand + t
        md5 = hashlib.md5()
        md5.update(md5Str.encode('utf-8'))
        token = md5.hexdigest()
        uuid = token[0:8] + '-'
        uuid = uuid + token[8:12] + '-'
        uuid = uuid + token[12:16] + '-'
        uuid = uuid + token[16:20] + '-'
        uuid = uuid + token[20:]
        return uuid

    def getData(self):
        return self.data

    def setToken(self, token):
        if(token):
            self.data['token'] = token
        return self

    def getToken(self):
        return self.data['token']


if __name__ == '__main__':

    directive = BaseDirective('TEXT')
    print(directive.getToken())
    pass


#!/usr/bin/env python3
# -*- coding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/30

"""
Session相关 暂未搞
"""

from dueros.Base import Base

class Session(Base):

    def __init__(self, data):

        super(Session,self).__init__()

        self.data = {}

        if 'attributes' in data:
            self.data = data['attributes']

        if 'sessionId' in data:
            self.sessionId = data['sessionId']
        else:
            self.sessionId = None

        if 'new' in data:
            self.isNew = data['new']
        else:
            self.isNew = False

    def clear(self):
        self.data = {}

    def toResponse(self):

        return {
            'attributes': self.data
        }

    def getData(self, field, default):
        if field in self.data:
            return self.data[field]

    def setData(self, field, value, default):

        self.data[field] = value


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""


from dueros.directive.Display.template.ListTemplate import ListTemplate
from dueros.directive.Display.template.ListTemplateItem import ListTemplateItem
class ListTemplate1(ListTemplate):

    def __init__(self):
        super(ListTemplate1, self).__init__('ListTemplate1')

if __name__ == '__main__':
    listTemplate = ListTemplate1()
    listTemplate.setToken('4234234')
    listTemplate.setBackGroundImage('https://skillstore.cdn.bcebos.com/icon/100/c709eed1-c07a-be4a-b242-0b0d8b777041.jpg')
    listTemplate.setTitle('托尔斯泰')

    item = ListTemplateItem()
    item.setToken('1')
    item.setPlainPrimaryText('p')
    item.setPlainSecondaryText('s')
    item.setImage('asfasdf')
    listTemplate.addItem(item)
    print(listTemplate.getData())
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""


class BotMonitorConfig:

    @staticmethod
    def getHost():
        return 'dueros-api.baidu.com'

    @staticmethod
    def getUploadUrl():
        return 'https://dueros-api.baidu.com/uploadmonitordata'
        #return 'http://127.0.0.1:8000'

    @staticmethod
    def getSdkVersion():
        return '1.0.0'

    @staticmethod
    def getSdkType():
        return 'python'

    @staticmethod
    def getUploadPort():
        return 443

    @staticmethod
    def getUploadPath():
        return '/uploadmonitordata'


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""

from dueros.directive.Display.template.BaseTemplate import BaseTemplate
from dueros.directive.Display.template.TextType import TextType
from dueros.directive.Display.template.TextContentPosition import TextContentPosition

class BodyTemplate1(BaseTemplate):

    def __init__(self):
        super(BodyTemplate1, self).__init__(['token','title','type'])
        self.setType('BodyTemplate1')
        pass

    def setPlainTextContent(self, text, position=TextContentPosition.BOTTOM_LEFT ):
        '''

        :param text:
        :param position:
        :return:
        '''
        textStructure = self.createTextStructure(text, TextType.PLAIN_TEXT)

        if textStructure:
            if not 'textContent' in self.data.keys():
                self.data['textContent'] = {}
            self.data['textContent']['text'] = textStructure.getData()
            if TextContentPosition.inEnum(position):
                self.data['textContent']['position'] = position.value
            else:
                self.data['textContent']['position'] = TextContentPosition.BOTTOM_LEFT

            return self

if __name__ == '__main__':

    bodytemplate = BodyTemplate1()
    bodytemplate.setTitle('呵呵')
    bodytemplate.setToken("tttt")
    bodytemplate.setBackGroundImage('htt[://///')
    bodytemplate.setPlainTextContent('bodyTemplate')
    print(bodytemplate.getData())
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""

from dueros.directive.Display.template.BaseTemplate import BaseTemplate

class BodyTemplate5(BaseTemplate):

    def __init__(self):
        super(BodyTemplate5, self).__init__(['token','title', 'type'])
        self.setType('BaseTemplate')
        self.data['images'] = []

    def addImages(self, url, widthPixels = '', heightPixel = ''):

        imageStructure = self.createImageStructure(url, widthPixels, heightPixel)
        if imageStructure:
            self.data['images'].append(imageStructure.getData())
        return self
    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

# description:
# author:jack
# create_time: 2017/12/30

import re
from dueros.card.TextCard import TextCard
from dueros.directive.BaseDirective import BaseDirective
from dueros.Base import Base

class Response(Base):
    """
    处理完后构建返回数据
    """

    def __init__(self, request, session, nlu):
        '''

        :param request:
        :param session:
        :param nlu:
        '''
        super(Response, self).__init__()
        self.request = request
        self.session = session
        self.nlu = nlu
        self.sourceType = self.request.getBotId()
        self.shouldEndSession = True
        self.needDetermine = False
        self.expectSpeech = False
        self.fallBack = False

    def setShouldEndSession(self, val):
        '''
        设置对话结束
        :param val:
        :return:
        '''
        if val:
            self.shouldEndSession = True
        else:
            self.shouldEndSession = False

    def defaultResult(self):

        return {
            'status': 0,
            'msg': ''
        }

    def build(self, data):
        '''
        构造response 返回结果
        :param data:
        data = {
            'card': card,
            'directives': directives,
            'outputSpeech': string,
            'reprompt': string
        }
        :return:
        '''

        if self.nlu and self.nlu.hasAsked():
            self.shouldEndSession = False

        if 'directives' in data:
            data['directives'] = data.get('directives')
        else:
            data['directives'] = []

        if 'card' in data:
            data['card'] = data.get('card')
        else:
            data['card'] = None

        if 'outputSpeech' in data:
            data['outputSpeech'] = data.get('outputSpeech')
        else:
            data['outputSpeech'] = None

        if 'resource' in data:
            data['resource'] = data.get('resource')
        else:
            data['resource'] = None

        if 'reprompt' in data:
            data['reprompt'] = data.get('reprompt')
        else:
            data['reprompt'] = None

        if 'directives' in data:
            directives = data.get('directives')
        else:
            directives = []

        if len(directives) > 0:
            directives = list(map(lambda value: value.getData(), list(filter(lambda value: isinstance(value, BaseDirective), directives))))

        if self.nlu:
            arr = self.nlu.toDirective()
            if(arr):
                directives.append(arr)

        if not data['outputSpeech'] and data['card'] and isinstance(data['card'], TextCard):
            data['outputSpeech'] = data['card'].getData()['content']

        if self.nlu:
            if self.nlu.toUpdateIntent():
                context = self.nlu.toUpdateIntent()
            else:
                context = {}
        else:
            context = {}

        ret = {
            "version": "2.0",
            "context": context,
            "session": self.session.toResponse(),
            "response": {
                "directives":  directives,
                "shouldEndSession": self.shouldEndSession,
                "card": data['card'].getData() if data['card'] else None,
                "resource": data['resource'],
                "outputSpeech": self.formatSpeech(data['outputSpeech']),
                "reprompt": {
			        "outputSpeech": self.formatSpeech(data['reprompt']),
		        }
            }
        }

        if self.needDetermine:
            ret['response']['needDetermine'] = self.needDetermine

        if self.expectSpeech:
            ret['response']['expectSpeech'] = self.expectSpeech

        if self.fallBack:
            ret['response']['fallBack'] = self.fallBack
        return ret

    def formatSpeech(self, mix):
        '''
        通过正则 判断是纯文本还是ssml
        :param mix:
        :return:
        '''
        if not mix or mix == 'null' or mix == '':
            return None

        result = {}
        if re.search(r'<speak>', mix):
            result = {
                "type": "SSML",
                "ssml": mix
            }
        else:
            result = {
                "type": "PlainText",
                "text": mix
            }
        return result


    def illegalRequest(self):

        return {
            'status': 1,
            'msg': '非法请求'
        }

    def setNeedDetermine(self):

        self.needDetermine = True

    def setExpectSpeech(self, expectSpeech):
        '''
        通过控制expectSpeech来控制麦克风开关
        :param expectSpeech:
        :return:
        '''

        self.expectSpeech = expectSpeech

    def setFallBack(self):
        '''
        表示本次返回的结果为兜底结果
        :return:
        '''

        self.fallBack = True

if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/31 上午12:24

"""
    desc:pass
"""


class __init__:

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""
from dueros.Intercept import Intercept
from dueros.card.TextCard import TextCard

class DuerSessionIntercept(Intercept):

    def __init__(self, tip = "非常抱歉，不明白你说的意思，已经取消了本次服务", threshold=2):
        self.tip = tip
        self.threshold = threshold

    def preprocess(self, bot):
        if(not self.threshold):
            return

        #NLU尝试slot提取，异常次数
        daException = bot.getSlots('da_system_not_understand')
        #bot 自身slot检查，不合法次数
        botException = bot.getSlots('bot_not_understand')
        count = 0
        if(daException):
            count = count + daException

        if(botException):
            count = count + botException

        if(count >= self.threshold):
            bot.clearSessionAttribute()
            bot.endDialog()
            card = TextCard(self.tip)
            return {
                'card': card
            }

if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/30

from dueros.card.BaseCard import BaseCard


class TextCard(BaseCard):
    """
    卡片形式展示
    """

    def __init__(self, content):
        '''
        文本卡片显示的content
        :param content:
        '''
        super(TextCard, self).__init__(['content'])
        self.data['type'] = "txt"
        self.data['content'] = "%s" % content

if __name__ == '__main__':

    textCard = TextCard('sdfasdfs')
    textCard.setContent('hehe')
    print(textCard.getData())
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-
# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""

from dueros.Bot import Bot
from dueros.directive.AudioPlayer.Stop import Stop
from dueros.directive.AudioPlayer.Play import Play
class Bot(Bot):
    '''
    自定义的BOT函数
    '''
    def launchRequest(self):
        '''
        打开调用名
        '''
        return {
            'outputSpeech': r'欢迎进入'
        }

    def audioPlay(self):
        '''
        播放指令
        '''
        directives = []
        directive = Play('http://m10.music.126.net/20180308193927/e9ca2860e7923536212ab420aad7b78d/ymusic/e18c/c0a0/ffef/1770e72b9dd1be8f814a566cab863c23.mp3')
        directives.append(directive)
        return {
            'directives' : directives,
            'outputSpeech': r'正在为你播放歌曲'
        }

    def audioStop(self):
        '''
        暂停指令
        '''
        directives = []
        directive = Stop()
        directives.append(directive)
        return {
            'directives' : directives,
            'outputSpeech': r'已经停止播放'
        }

    def playBackStartedEvent(self, event):
        '''
        处理事件上报示例
        '''
        offset = event['offsetInMilliSeconds']
        directives = []
        directive = Play('http://m10.music.126.net/20180308193927/e9ca2860e7923536212ab420aad7b78d/ymusic/e18c/c0a0/ffef/1770e72b9dd1be8f814a566cab863c23.mp3', Play.REPLACE_ALL)
        directives.append(directive)
        return {
            'outputSpeech': r'已处理端上报事件',
            'directives' : directives,
        }

    def playBackNearlyFinished(self, event):
        '''
        处理事件上报示例
        '''
        offset = event['offsetInMilliSeconds']
        return {
            'outputSpeech': r'已处理端上报事件'
        }

    def __init__(self, data):
        '''
        构造函数
        '''
        super(Bot, self).__init__(data)

        self.addLaunchHandler(self.launchRequest)
        #给端下发指令
        self.addIntentHandler('audio_play_intent', self.audioPlay)
        self.addIntentHandler('audio_stop_intent', self.audioStop)
        #处理端上报事件
        self.addEventListener('AudioPlayer.PlaybackStarted', self.playBackStartedEvent)
        self.addEventListener('AudioPlayer.PlaybackNearlyFinished', self.playBackStartedEvent)


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""
from cgi import parse_qs, escape
import json
from dueros.samples.Bot import Bot

def application(environ, start_response):
    try:
        request_body_size = int(environ.get('CONTENT_LENGTH', 0))
    except(ValueError):
        request_body_size = 0
    request_body = environ['wsgi.input'].read(request_body_size).decode('utf-8')
    print('request_body = %s\n' % request_body)
    if not request_body:
        return ['未获取到请求数据']

    bot = Bot(request_body)
    #添加错误回调方法
    bot.setCallBack(callback)

    #验证签名enableVerifyRequestSign  disableVerifyRequestSign 关闭验证签名
    # bot.initCertificate(environ).enableVerifyRequestSign()
    # bot.initCertificate(environ).disableVerifyRequestSign()

    body_str = bot.run()

    body = body_str.encode('utf-8')

    response_headers = [('Content-Type', 'application/json'),
                        ('Content-Length', str(len(body)))]
    status = '200 OK'

    start_response(status, response_headers)

    return [body]

def callback(data):
    print(data)
def askContol():
        response = '''
        {
        "session": {
            "attributes": {}
        },
        "version": "2.0",
        "response": {
            "resource": null,
            "outputSpeech": {
                "text": "请告诉您的指令，比如调小空调风速、设置温度为30度",
                "type": "PlainText"
            },
            "directives": [
                {
                "updatedIntent": {
                    "slots": {
                        "deviceName": {
                            "name": "deviceName",
                            "value": "kongtiao"
                        },
                        "city": {
                            "score": 0,
                            "name": "city",
                            "value": "beijing",
                            "confirmationStatus": "NONE"
                        }
                    },
                    "name": "dueros.device_interface.smart_device.control"
                },
                "type": "Dialog.ElicitSlot",
                "slotToElicit": "deviceName"
                }
            ],
            "reprompt": null,
            "shouldEndSession": false,
            "card": {
                "content": "请告诉您的指令，比如调小空调风速、设置温度为30度",
                "type": "txt"
            }
        },
        "context": {
            "intent": {
                "slots": {
                    "deviceName": {
                        "name": "deviceName",
                        "value": "kongtiao"
                    },
                    "city": {
                        "score": 0,
                        "name": "city",
                        "value": "beijing",
                        "confirmationStatus": "NONE"
                    }
                },
                "score": 100,
                "name": "dueros.device_interface.smart_device.control",
                "confirmationStatus": "NONE"
            }
        }
    }'''
        return response




#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""

class TextStructure:

    def __init__(self):
        super(TextStructure, self).__init__()
        self.data = {}
        self.setType('PlainText')

    def setType(self, type):

        if type:
            self.data['type'] = type

    def setText(self, text):

        if text:
            self.data['text'] = text

    def getData(self):
        return self.data
    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""

from dueros.directive.AudioPlayer.Control.Button import Button

class PreviousButton(Button):

    def __init__(self):
        super(PreviousButton, self).__init__('PREVIOUS')


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

# description:
# author:jack
# create_time: 2017/12/31

"""
    desc:pass
"""

import logging
from dueros.Bot import Bot
from dueros.card.TextCard import TextCard
from dueros.directive.Display.template.BodyTemplate1 import BodyTemplate1
from dueros.directive.Display.RenderTemplate import RenderTemplate
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger('bot')
import sys


class BotTest(Bot):

    def launchRequest(self):
        return {
            'card': TextCard(r'欢迎使用家居控制!请告诉我您要查找什么智能设备，比如查找我的空调'),
            'outputSpeech': r'<speak>欢迎使用家居控制!请告诉我您要查找什么智能设备，比如查找我的空调</speak>'
        }

    def searchRequest(self):

        self.ask('deviceName')

        # card = TextCard('您要查找什么智能设备呢? 比如"查找我的空调"')
        # card.addCueWords("百度")
        # card.addCueWords("百度")
        # card.addCueWords("百度")
        # card.setAnchor("http://www.baidu.com", "百度")

        # renderTemplate = RenderTemplate()
        bodyTemplate = BodyTemplate1()
        bodyTemplate.setToken('token')
        bodyTemplate.setTitle('托尔斯泰的格言')
        bodyTemplate.setBackGroundImage('https://skillstore.cdn.bcebos.com/icon/100/c709eed1-c07a-be4a-b242-0b0d8b777041.jpg')
        bodyTemplate.setPlainTextContent('拖尔斯泰-理想的书籍是智慧的钥匙')
        renderTemplate = RenderTemplate(bodyTemplate)

        # renderTemplate.setTemplate(bodyTemplate)
        renderTemplate.setToken("renderTemplate")
        return {
            'directives': [renderTemplate],
            'outputSpeech': '<speak>您要查找什么智能设备呢? 比如"查找我的空调"</speak>'
        }

    def controlRequest(self):
        self.ask('deviceName')
        # deviceName = self.getSlots('deviceName')
        # print('deviceName %s' % (deviceName))
        return {
            'card': TextCard('请告诉您的指令，比如调小空调风速、设置温度为30度'),
            'outputSpeech': '请告诉您的指令，比如调小空调风速、设置温度为30度'
        }

    def inquiry(self):
        return {
            'directives': TextCard('请告诉您的指令，比如调小空调风速、设置温度为30度'),
            'outputSpeech': '请告诉您的指令，比如调小空调风速、设置温度为30度'
        }

    def __init__(self, data):
        super(BotTest, self).__init__(data)

        self.addLaunchHandler(self.launchRequest)

        self.addIntentHandler('dueros.device_interface.smart_device.control', self.controlRequest)

        self.addIntentHandler('dueros.device_interface.smart_device.search', self.searchRequest)
        # self.addIntentHandler('inquiry', self.inquiry)
    pass

if __name__ == '__main__':

    def launchData():
        with open("./json/launch.json", 'r', encoding='utf-8') as load_f:
            return load_f.read()

    def searchData():
        with open("./json/search2.json", 'r', encoding='utf-8') as load_f:
            return load_f.read()

    def controlData():
        with open("./json/control.json", 'r', encoding='utf-8') as load_f:
            return load_f.read()


    def controlData2():
        with open("./json/a.json", 'r', encoding='utf-8') as load_f:
            return load_f.read()


    def tt(data):
        print(data)
    data = searchData()
    bot = BotTest(data)
    bot.setCallBack(tt)
    bot.run()
    #
    # rand = str(random.randint(0, 9999999999))
    # t = str(round(time.time() * 1000))
    # md5 = hashlib.md5()
    # md5Str = rand + t
    # print("md5Str = %s, encode = %s ", (md5Str,md5Str.encode('utf-8')))
    # md5.update(md5Str.encode('utf-8'))
    # token = md5.hexdigest()
    # uuid = token[0:8] + '-'
    # uuid = uuid + token[8:12] + '-'
    # uuid = uuid + token[12:16] + '-'
    # uuid = uuid + token[16:20] + '-'
    # uuid = uuid + token[20:]
    print(sys.version_info[0])
    #while True:
    #    print("====")
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""

from dueros.directive.Display.template.TextImageTemplate import TextImageTemplate


class BodyTemplate2(TextImageTemplate):

    def __init__(self):
        super(BodyTemplate2, self).__init__('BodyTemplate2')
        pass

if __name__ == '__main__':
    bodytemplate = BodyTemplate2()
    bodytemplate.setTitle('呵呵')
    bodytemplate.setToken("tttt")
    bodytemplate.setBackGroundImage('http://adfasdf')
    bodytemplate.setPlainContent('bodyTemplate')
    print(bodytemplate.getData())
    pass



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/31 上午12:21

import json
from dueros.card.BaseCard import BaseCard
from dueros.card.ListCardItem import ListCardItem


class ListCard(BaseCard):

    def __init__(self):
        super(ListCard, self).__init__()
        self.data['type'] = 'list'

    def addItem(self, listCardIetm):

        if isinstance(listCardIetm, ListCardItem):

            if not 'list' in self.data:
                self.data['list'] = []
            self.data['list'].append(listCardIetm.getData())
        return self


if __name__ == '__main__':

    listCardItem1 = ListCardItem()
    listCardItem1.setTitle('baidu')
    listCardItem1.setUrl("http://www.baidu.com")
    listCardItem1.setImage("http://www.baidu.com")
    listCardItem1.setContent("http://www.baidu.com")

    print(id(listCardItem1))
    listCardItem2 = ListCardItem()
    listCardItem2.setTitle("百度2")
    print(id(listCardItem2))

    listCardItem3 = ListCardItem()
    listCardItem3.setTitle("百度3")
    print(id(listCardItem3))

    listCard = ListCard()
    listCard.addItem(listCardItem1)
    listCard.addItem(listCardItem2)
    listCard.addItem(listCardItem3)

    print(json.dumps(listCard.getData()))

    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3


class Intercept:

    def preprocess(self, bot):
        '''

        :param bot:
        :return:
        如果返回非null，跳过后面addHandler，addEventListener添加的回调
        '''
        pass

    def postprocess(self, bot, result):
        '''
        在调用response->build 之前统一对handler的输出结果进行修改
        :param bot:
        :param result:  []
        :return:[]
        '''

        return result

if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2017/12/31

from dueros.directive.BaseDirective import BaseDirective


class LaunchBrowser(BaseDirective):
    """
        用于调用浏览器指令的类
    """
    def __init__(self, url):

        super(LaunchBrowser, self).__init__('WebBrowser.LaunchBrowser')
        self.data['url'] = url
        self.data['token'] = self.getToken()

if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/6

"""
    desc:pass
"""

from setuptools import setup
from setuptools import find_packages

setup(
    name='dueros-bot',
    version='0.2.5',
    author='jack',
    author_email='mupdf@sina.com',
    description='Third party DuerOS Bot Python SDK',
    keywords='DuerOS Bot',
    url='https://github.com/jokenwang/bot-sdk-python',
    packages=find_packages(),
    install_requires=[
        'pycrypto>=2.6.1',
        'requests',
	 'pyOpenSSL'
    ]
)



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-
# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""

from dueros.Bot import Bot
class Bot(Bot):
    def launchRequest(self):
        '''
        打开调用名
        '''
        self.waitAnswer()
        return {
            'outputSpeech': r'欢迎进入'
        }

    def getTaxSlot(self):
        '''
        获取槽位及逻辑处理
        '''
        num = self.getSlots('sys.number')
        city = self.getSlots('sys.city')
        if num and not city:
            self.nlu.ask('sys.city')
            return {
                'reprompt': r'你所在的城市是那里呢',
                'outputSpeech': r'你所在的城市是那里呢'
            }

        if city and not num:
            self.nlu.ask('sys.number')
            return {
                'reprompt': r'你的税前工资是多少呢',
                'outputSpeech': r'你的税前工资是多少呢'
            }
         
        computeType = self.getSlots('compute_type')
        if not computeType:
            self.nlu.ask('compute_type')
            return {
                'outputSpeech': r'你要查询什么税种呢'
            }
        else:
            taxNum = self.computeType(num, city)
            return {
                'outputSpeech': r'你需要缴纳' + str(taxNum)
            }
    
    def computeType(self, num, city):
        '''
        调用接口计算个税
        '''
        return 100


    def __init__(self, data):
        super(Bot, self).__init__(data)
        self.addLaunchHandler(self.launchRequest)
        self.addIntentHandler('personal_income_tax.inquiry', self.getTaxSlot)


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/27

"""
    desc:pass
"""
from dueros.directive.Display.template.BaseTemplate import BaseTemplate
from dueros.directive.Display.template.TextType import TextType

class TextImageTemplate(BaseTemplate):

    def __init__(self, type):
        super(TextImageTemplate, self).__init__(['token','title', 'type'])
        self.setType(type)

    def setImage(self, url, widthPixels = '', heightPixels = ''):

        imageStructure = self.createImageStructure(url, widthPixels, heightPixels)
        if imageStructure:
            self.data['image'] = imageStructure.getData()

    def setPlainContent(self, text):
        textStructure = self.createTextStructure(text, TextType.PLAIN_TEXT)
        if textStructure:
            self.data['content'] = textStructure.getData()
        pass


if __name__ == '__main__':


    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""
from dueros.directive.AudioPlayer.Control.RadioButton import RadioButton
from dueros.directive.AudioPlayer.Control.ThumbsUpDownButtonEnum import ThumbsUpDownButtonEnum

class ThumbsUpDownButton(RadioButton):

    def __init__(self, selectedValue=ThumbsUpDownButtonEnum.THUMBS_UP.value):
        super(ThumbsUpDownButton, self).__init__('THUMBS_UP_DOWN', selectedValue)

    def setSelectedValue(self, selectedValue=ThumbsUpDownButtonEnum.THUMBS_UP):
        if ThumbsUpDownButtonEnum.inEnum(selectedValue):
            self.data['selectedValue'] = selectedValue.value
        else:
            self.data['selectedValue'] = ThumbsUpDownButtonEnum.THUMBS_UP.value

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/1/3

"""
    desc:pass
"""
import os
import fcntl
import hashlib
import OpenSSL
import urllib.request
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA
from base64 import b64encode, b64decode
from dueros.Base import Base

class Certificate(Base):

    def __init__(self, environ, requestBody, privateKeyContent = ""):
        '''
        私钥内容,使用统计功能必须要提供
        :param environ: 环境上下文
        :param requestBody:
        :param privateKeyContent:
        '''

        super(Certificate, self).__init__()

        self.environ = environ
        self.data = requestBody
        self.privateKey = privateKeyContent
        self.verifyRequestSign = False

    def enableVerifyRequestSign(self):
        self.verifyRequestSign = True

    def disableVerifyRequestSign(self):
        self.verifyRequestSign = False

    def getRequestPublicKey(self):

        filename = self.environ['HTTP_SIGNATURECERTURL']
        if not filename:
            return
        md5 = hashlib.md5()
        md5.update(filename.encode('utf-8'))
        cache = os.getcwd() + os.path.sep + md5.hexdigest()
        content = ''
        if not os.path.exists(cache):
            content = urllib.request.urlopen(filename).read()
            if not content:
                return
            with open(cache, 'w') as f:
                fcntl.flock(f, fcntl.LOCK_EX)
                f.write(content)
                fcntl.flock(f, fcntl.LOCK_UN)
        content = self.getFileContentSafety(cache)
        return self.getPublicKeyFromX509(content)

    def verifyRequest(self):
        '''
        数据验证
        :return:
        '''
        if not self.verifyRequestSign:
            return True

        publicKey = self.getRequestPublicKey()

        if not publicKey or not self.data:
            return False

        key = RSA.importKey(publicKey)
        if key:
            digest = SHA.new()
            digest.update(self.data)
            verifier = PKCS1_v1_5.new(key)
            if verifier.verify(digest, b64decode(self.getRequestSign())):
                return True
            return False
        return False

    def getSign(self, content):
        '''
        生成签名
        :param content: 待签名内容
        :return:
        '''
        if not self.privateKey or not content:
            return False

        rsakey = RSA.importKey(self.privateKey)
        if rsakey:
            digest = SHA.new()
            digest.update(content)
            signer = PKCS1_v1_5.new(rsakey)
            signature = signer.sign(digest)
            return b64encode(signature)
        else:
            return False

    def getRequestSign(self):
        return self.environ['HTTP_SIGNATURE']
    
    def getPublicKeyFromX509(self, content):
        '''
        获取publicKey
        :param X509 content
        :return publicKey
        '''
        x509 = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, content)
        pk = x509.get_pubkey()
        publicKey = OpenSSL.crypto.dump_publickey(OpenSSL.crypto.FILETYPE_PEM, pk)
        return publicKey

    def getFileContentSafety(self, filename):
        '''
        获取文件内容
        :param filename
        :return content
        '''
        with open(filename, 'r') as f:
            fcntl.flock(f,fcntl.LOCK_SH)
            content = f.read()
            return content
            fcntl.flock(f,fcntl.LOCK_UN)

if __name__ == '__main__':

    priKey = '''-----BEGIN RSA PRIVATE KEY-----
MIICXQIBAAKBgQDKoeRzRVf8WoRSDYYqUzThpYCr90jfdFwTSXHJ526K8C6TEwdT
UA+CFPQPRUg9jrYgFcown+J2myzO8BRLynD+XHb9ilLb49Mqk2CvDt/yK32lgHv3
QVx14Dpb6h8isjncSF965fxBxlHGbvPwnHkJ9etRIYdYV3QpYohFszH3wQIDAQAB
AoGAFhKqkw/ztK6biWClw8iKkyX3LURjsMu5F/TBK3BFb2cYe7bv7lhjSBVGPL+c
TfBU0IvvGXrhLXBb4jLu0w67Xhggwwfc86vlZ8eLcrmYVat7N6amiBmYsw20GViU
UFmePbo1G2BXqMA43JxqbIQwOLZ03zdw6GHj6EVlx369IAECQQD4K2R3K8ah50Yz
LhF7zbYPIPGbHw+crP13THiYIYkHKJWsQDr8SXoNQ96TQsInTXUAmF2gzs/AwdQg
gjIJ/dmBAkEA0QarqdWXZYbse1XIrQgBYTdVH9fNyLs1e1sBmNxlo4QMm/Le5a5L
XenorEjnpjw5YpEJFDS4ijUI3dSzylC+QQJARqcD6TGbUUioobWB4L9GD7SPVFxZ
c3+EgcxRoO4bNuCFDA8VO/InP1ONMFuXLt1MbCj0ru1yFCyamc63NEUDAQJBALt7
PjGgsKCRuj6NnOcGDSbDWIitKZhnwfqYkAApfsiBQkYGO0LLaDIeAWG2KoCB9/6e
lAQZnYPpOcCubWyDq4ECQQCrRDf0gVjPtipnPPS/sGN8m1Ds4znDDChhRlw74MI5
FydvHFumChPe1Dj2I/BWeG1gA4ymXV1tE9phskV3XZfq
-----END RSA PRIVATE KEY-----'''
    pubKey = '''-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDKoeRzRVf8WoRSDYYqUzThpYCr
90jfdFwTSXHJ526K8C6TEwdTUA+CFPQPRUg9jrYgFcown+J2myzO8BRLynD+XHb9
ilLb49Mqk2CvDt/yK32lgHv3QVx14Dpb6h8isjncSF965fxBxlHGbvPwnHkJ9etR
IYdYV3QpYohFszH3wQIDAQAB
-----END PUBLIC KEY-----'''

    data = 'partner="2088701924089318"&seller="774653@qq.com"&out_trade_no="123000"&subject="123456"&body="2010新款NIKE 耐克902第三代板鞋 耐克男女鞋 386201 白红"&total_fee="0.01"¬ify_url="http://notify.java.jpxx.org/index.jsp'
    def sign(data):
        key = RSA.importKey(priKey)
        digest = SHA.new()
        digest.update(data.encode('utf-8'))
        signer = PKCS1_v1_5.new(key)
        signature = signer.sign(digest)
        return b64encode(signature)


    def verify(data, signature):
        key = RSA.importKey(pubKey)
        digest = SHA.new()
        digest.update(data.encode('utf-8'))
        verifier = PKCS1_v1_5.new(key)
        if verifier.verify(digest, b64decode(signature)):
            return True
        return False

    signData = sign(data)
    print(sign(data))
    print(verify(data, signData))
    pass



#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""

class ImageStructure:

    def __init__(self):
        self.data = {}
        super(ImageStructure, self).__init__()


    def setUrl(self, url):
        if url:
            self.data['url'] = url

    def setWidthPixels(self, width):
        if width:
            self.data['widthPixels'] = width

    def setHeightPixels(self, height):
        if height:
            self.data['heightPixels'] = height

    def getData(self):
        return self.data

if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/4/15

"""
    desc:pass
"""
import dueros.Log as Log
from dueros.Constants import constants
import logging

class Base:

    def __init__(self):
        Log.init_log(constants.LOG_PATH)


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/2/26

"""
    desc:pass
"""

import os
import logging
import logging.handlers

def init_log(log_path, level=logging.INFO, when="D", backup=7,
             format="%(levelname)s: %(asctime)s: %(filename)s:%(lineno)d * %(thread)d %(message)s",
             datefmt="%m-%d %H:%M:%S"):
    """
    init_log - initialize log module

    Args:
      log_path      - Log file path prefix.
                      Log data will go to two files: log_path.log and log_path.log.wf
                      Any non-exist parent directories will be created automatically
      level         - msg above the level will be displayed
                      DEBUG < INFO < WARNING < ERROR < CRITICAL
                      the default value is logging.INFO
      when          - how to split the log file by time interval
                      'S' : Seconds
                      'M' : Minutes
                      'H' : Hours
                      'D' : Days
                      'W' : Week day
                      default value: 'D'
      format        - format of the log
                      default format:
                      %(levelname)s: %(asctime)s: %(filename)s:%(lineno)d * %(thread)d %(message)s
                      INFO: 12-09 18:02:42: log.py:40 * 139814749787872 HELLO WORLD
      backup        - how many backup file to keep
                      default value: 7

    Raises:
        OSError: fail to create log directories
        IOError: fail to open log file
    """
    formatter = logging.Formatter(format, datefmt)
    logger = logging.getLogger()
    logger.setLevel(level)

    dir = os.path.dirname(log_path)
    if not os.path.isdir(dir):
        os.makedirs(dir)

    handler = logging.handlers.TimedRotatingFileHandler(log_path + ".log",
                                                        when=when,
                                                        backupCount=backup)
    handler.setLevel(level)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    handler = logging.handlers.TimedRotatingFileHandler(log_path + ".log.wf",
                                                        when=when,
                                                        backupCount=backup)
    handler.setLevel(logging.WARNING)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

if __name__ == '__main__':

    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/28

"""
    desc:pass
"""


class BaseButton:

    def __init__(self, type, name):
        self.data = {}
        self.data['type'] = type
        self.data['name'] = name

    def getData(self):
        return self.data

    pass


if __name__ == '__main__':
    pass


#!/usr/bin/env python3
# -*- encoding=utf-8 -*-

# description:
# author:jack
# create_time: 2018/5/26

"""
    desc:pass
"""

from dueros.directive.Display.template.TextImageTemplate import TextImageTemplate


class BodyTemplate4(TextImageTemplate):

    def __init__(self):
        super(BodyTemplate4, self).__init__('BodyTemplate4')


if __name__ == '__main__':
    pass